@@ -2,12 +2,15 @@
 # -*- coding: utf-8 -*-
 
 import argparse
+import json
 import os
 import sys
 
 from rich.panel import Panel
+from rich.terminal_theme import MONOKAI
 from vdb.lib import config as config
 from vdb.lib import db as dbLib
+from vdb.lib.aqua import AquaSource
 from vdb.lib.gha import GitHubSource
 from vdb.lib.nvd import NvdSource
 from vdb.lib.osv import OSVSource
@@ -27,6 +30,11 @@ from depscan.lib.config import license_data_dir, spdx_license_list
 from depscan.lib.license import build_license_data, bulk_lookup
 from depscan.lib.logger import LOG, console
 
+try:
+    os.environ["PYTHONIOENCODING"] = "utf-8"
+except Exception:
+    pass
+
 at_logo = """
   ___            _____ _                    _
  / _ \          |_   _| |                  | |
@@ -41,7 +49,7 @@ at_logo = """
 
 def build_args():
     """
-    Constructs command line arguments for the vulndb tool
+    Constructs command line arguments for the depscan tool
     """
     parser = argparse.ArgumentParser(
         description="Fully open-source security and license audit for application dependencies and container images based on known vulnerabilities and advisories."
@@ -60,6 +68,13 @@ def build_args():
         dest="cache",
         help="Cache vulnerability information in platform specific user_data_dir",
     )
+    parser.add_argument(
+        "--cache-os",
+        action="store_true",
+        default=False,
+        dest="cache_os",
+        help="Cache OS vulnerability information in platform specific user_data_dir",
+    )
     parser.add_argument(
         "--sync",
         action="store_true",
@@ -70,9 +85,9 @@ def build_args():
     parser.add_argument(
         "--suggest",
         action="store_true",
-        default=False,
+        default=True,
         dest="suggest",
-        help="Suggest appropriate fix version for each identified vulnerability.",
+        help="DEPRECATED: Suggest is the default mode for determining fix version.",
     )
     parser.add_argument(
         "--risk-audit",
@@ -91,6 +106,7 @@ def build_args():
         "-t",
         "--type",
         dest="project_type",
+        default=os.getenv("DEPSCAN_PROJECT_TYPE"),
         help="Override project type if auto-detection is incorrect",
     )
     parser.add_argument(
@@ -108,7 +124,13 @@ def build_args():
         "-o",
         "--report_file",
         dest="report_file",
-        help="Report filename with directory",
+        help="DEPRECATED. Use reports directory since multiple files are created. Report filename with directory",
+    )
+    parser.add_argument(
+        "--reports-dir",
+        default=os.getenv("DEPSCAN_REPORTS_DIR", os.path.join(os.getcwd(), "reports")),
+        dest="reports_dir",
+        help="Reports directory",
     )
     parser.add_argument(
         "--no-error",
@@ -122,7 +144,14 @@ def build_args():
         action="store_true",
         default=False,
         dest="no_license_scan",
-        help="DREPRECATED: dep-scan doesn't perform license scanning by default",
+        help="DEPRECATED: dep-scan doesn't perform license scanning by default",
+    )
+    parser.add_argument(
+        "--deep",
+        action="store_true",
+        default=False,
+        dest="deep_scan",
+        help="Perform deep scan by passing this --deep argument to cdxgen. Useful while scanning docker images and OS packages.",
     )
     return parser.parse_args()
 
@@ -140,7 +169,7 @@ def scan(db, project_type, pkg_list, suggest_mode):
         LOG.debug("Empty package search attempted!")
     else:
         LOG.info("Scanning {} oss dependencies for issues".format(len(pkg_list)))
-    results, pkg_aliases = utils.search_pkgs(db, project_type, pkg_list)
+    results, pkg_aliases, purl_aliases = utils.search_pkgs(db, project_type, pkg_list)
     # pkg_aliases is a dict that can be used to find the original vendor and package name
     # This way we consistently use the same names used by the caller irrespective of how
     # the result was obtained
@@ -178,22 +207,24 @@ def scan(db, project_type, pkg_list, suggest_mode):
             LOG.debug(
                 "Re-checking our suggestion to ensure there are no further vulnerabilities"
             )
-            override_results, _ = utils.search_pkgs(db, project_type, sug_pkg_list)
+            override_results, _, _ = utils.search_pkgs(db, project_type, sug_pkg_list)
             if override_results:
                 new_sug_dict = suggest_version(override_results)
                 LOG.debug("Received override results: {}".format(new_sug_dict))
                 for nk, nv in new_sug_dict.items():
                     sug_version_dict[nk] = nv
-    return results, pkg_aliases, sug_version_dict
+    return results, pkg_aliases, sug_version_dict, purl_aliases
 
 
 def summarise(
     project_type,
     results,
     pkg_aliases,
+    purl_aliases,
     sug_version_dict,
     scoped_pkgs={},
     report_file=None,
+    bom_file=None,
     console_print=True,
 ):
     """
@@ -201,6 +232,7 @@ def summarise(
     :param project_type: Project type
     :param results: Scan or audit results
     :param pkg_aliases: Package aliases used
+    :param purl_aliases: Package URL to package name aliase
     :param sug_version_dict: Dictionary containing version suggestions
     :param scoped_pkgs: Dict containing package scopes
     :param report_file: Output report file
@@ -215,12 +247,32 @@ def summarise(
             project_type,
             results,
             pkg_aliases,
+            purl_aliases,
             sug_version_dict,
             scoped_pkgs,
             report_file,
         )
     if console_print:
-        print_results(project_type, results, pkg_aliases, sug_version_dict, scoped_pkgs)
+        pkg_vulnerabilities = print_results(
+            project_type,
+            results,
+            pkg_aliases,
+            purl_aliases,
+            sug_version_dict,
+            scoped_pkgs,
+        )
+        if pkg_vulnerabilities and bom_file:
+            vex_file = bom_file.replace(".json", ".vex.json")
+            try:
+                with open(bom_file) as fp:
+                    bom_data = json.load(fp)
+                    if bom_data:
+                        bom_data["vulnerabilities"] = pkg_vulnerabilities
+                        with open(vex_file, mode="w") as vexfp:
+                            json.dump(bom_data, vexfp)
+                            LOG.info(f"VEX file {vex_file} generated successfully")
+            except Exception:
+                LOG.warn("Unable to generate VEX file for this scan")
     summary = analyse(project_type, results)
     return summary
 
@@ -232,7 +284,7 @@ def main():
     src_dir = args.src_dir_image
     if not src_dir:
         src_dir = os.getcwd()
-    reports_base_dir = src_dir
+    reports_dir = args.reports_dir
     # Detect the project types and perform the right type of scan
     if args.project_type:
         project_types_list = args.project_type.split(",")
@@ -240,21 +292,14 @@ def main():
         project_types_list = ["bom"]
     else:
         project_types_list = utils.detect_project_type(src_dir)
-    if (
-        "docker" in project_types_list
-        or "podman" in project_types_list
-        or "container" in project_types_list
-        or "binary" in project_types_list
-    ):
-        reports_base_dir = os.getcwd()
     db = dbLib.get()
-    run_cacher = args.cache
+    run_cacher = args.cache or args.cache_os
     areport_file = (
         args.report_file
         if args.report_file
-        else os.path.join(reports_base_dir, "reports", "depscan.json")
+        else os.path.join(reports_dir, "depscan.json")
     )
-    reports_dir = os.path.dirname(areport_file)
+    html_file = areport_file.replace(".json", ".html")
     # Create reports directory
     if reports_dir and not os.path.exists(reports_dir):
         os.makedirs(reports_dir, exist_ok=True)
@@ -285,12 +330,20 @@ def main():
             bom_file = args.bom
             creation_status = True
         else:
-            bom_file = os.path.join(reports_dir, "bom-" + project_type + ".json")
-            creation_status = create_bom(project_type, bom_file, src_dir)
+            bom_file = report_file.replace("depscan-", "sbom-")
+            creation_status = create_bom(
+                project_type, bom_file, src_dir, args.deep_scan
+            )
         if not creation_status:
             LOG.debug("Bom file {} was not created successfully".format(bom_file))
             continue
-        LOG.debug("Scanning using the bom file {}".format(bom_file))
+        LOG.info("Scanning using the bom file {}".format(bom_file))
+        if not args.bom:
+            LOG.info(
+                "To improve performance, cache this bom file and invoke depscan with --bom {} instead of -i".format(
+                    bom_file
+                )
+            )
         pkg_list = get_pkg_list(bom_file)
         if not pkg_list:
             LOG.debug("No packages found in the project!")
@@ -363,12 +416,12 @@ def main():
             except Exception as e:
                 LOG.error("Remote audit was not successful")
                 LOG.error(e)
-                results = None
+                results = []
         # In case of docker, check if there are any npm packages that can be audited remotely
         if project_type in ("podman", "docker"):
             npm_pkg_list = get_pkg_by_type(pkg_list, "npm")
             if npm_pkg_list:
-                LOG.debug(f"No of packages {len(npm_pkg_list)}")
+                LOG.debug(f"No of npm packages {len(npm_pkg_list)}")
                 try:
                     audit_results = audit("nodejs", npm_pkg_list, report_file)
                     if audit_results:
@@ -387,6 +440,15 @@ def main():
         if os.environ.get("GITHUB_TOKEN"):
             sources_list.insert(0, GitHubSource())
         if run_cacher:
+            if (
+                args.cache_os
+                or args.deep_scan
+                or project_type in ("docker", "podman", "yaml-manifest", "os")
+            ):
+                sources_list.insert(0, AquaSource())
+                LOG.info(
+                    "OS Vulnerability database would be downloaded for the first time. This would take a few minutes ..."
+                )
             for s in sources_list:
                 LOG.debug("Refreshing {}".format(s.__class__.__name__))
                 s.refresh()
@@ -406,7 +468,7 @@ def main():
                 src_dir, project_type
             )
         )
-        vdb_results, pkg_aliases, sug_version_dict = scan(
+        vdb_results, pkg_aliases, sug_version_dict, purl_aliases = scan(
             db, project_type, pkg_list, args.suggest
         )
         if vdb_results:
@@ -416,15 +478,18 @@ def main():
             project_type,
             results,
             pkg_aliases,
+            purl_aliases,
             sug_version_dict,
             scoped_pkgs,
             report_file,
+            bom_file,
             True,
         )
         if summary and not args.noerror and len(project_types_list) == 1:
             # Hard coded build break logic for now
-            if summary.get("CRITICAL") > 0:
+            if summary.get("CRITICAL", 0) > 0:
                 sys.exit(1)
+    console.save_html(html_file, theme=MONOKAI)
 
 
 if __name__ == "__main__":
