@@ -2,6 +2,7 @@ import json
 import os
 import shutil
 import subprocess
+import sys
 import xml
 from urllib.parse import unquote_plus
 
@@ -51,7 +52,6 @@ def parse_bom_ref(bomstr, licenses=None):
     tmpl = bomstr.split("/")
     vendor = ""
     name_ver = []
-
     if len(tmpl) == 2:
         # Just name and version
         vendor = tmpl[0]
@@ -109,18 +109,22 @@ def get_package(componentEle, licenses):
     """ """
     bom_ref = componentEle.attrib.get("bom-ref")
     pkg = {"licenses": licenses, "vendor": "", "name": "", "version": "", "scope": ""}
-    if bom_ref:
+    if bom_ref and "/" in bom_ref:
         pkg = parse_bom_ref(bom_ref, licenses)
     for ele in componentEle.iter():
         if ele.tag.endswith("group") and ele.text:
             pkg["vendor"] = ele.text
-        elif ele.tag.endswith("name") and ele.text and not pkg["name"]:
+        if ele.tag.endswith("name") and ele.text and not pkg["name"]:
             pkg["name"] = ele.text
         if ele.tag.endswith("version") and ele.text:
             version = ele.text
             if version.startswith("v"):
                 version = version[1:]
             pkg["version"] = version
+        if ele.tag.endswith("purl") and ele.text and not pkg.get("vendor"):
+            purl = ele.text
+            namespace = purl.split("/")[0].replace("pkg:", "")
+            pkg["vendor"] = namespace
     return pkg
 
 
@@ -170,7 +174,7 @@ def get_pkg_list(xmlfile):
                     if ele.tag.endswith("component"):
                         licenses = get_licenses(ele)
                         pkgs.append(get_package(ele, licenses))
-    except xml.etree.ElementTree.ParseError as pe:
+    except Exception as pe:
         LOG.debug("Unable to parse {} {}".format(xmlfile, pe))
         LOG.warning(
             "Unable to produce Software Bill-of-Materials for this project. Execute the scan after installing the dependencies!"
@@ -191,7 +195,15 @@ def get_pkg_by_type(pkg_list, pkg_type):
     ]
 
 
-def create_bom(project_type, bom_file, src_dir="."):
+def resource_path(relative_path):
+    try:
+        base_path = sys._MEIPASS
+    except Exception:
+        base_path = os.path.dirname(__file__)
+    return os.path.join(base_path, relative_path)
+
+
+def create_bom(project_type, bom_file, src_dir=".", deep=False):
     """Method to create BOM file by executing cdxgen command
 
     :param project_type: Project type
@@ -202,16 +214,32 @@ def create_bom(project_type, bom_file, src_dir="."):
     """
     cdxgen_cmd = os.environ.get("CDXGEN_CMD", "cdxgen")
     if not shutil.which(cdxgen_cmd):
-        LOG.warning(
-            "{} command not found. Please install using npm install @appthreat/cdxgen or set PATH variable".format(
-                cdxgen_cmd
+        local_bin = resource_path(
+            os.path.join(
+                "local_bin", "cdxgen.exe" if sys.platform == "win32" else "cdxgen"
             )
         )
-        return False
+        if not os.path.exists(local_bin):
+            LOG.warning(
+                "{} command not found. Please install using npm install @appthreat/cdxgen or set PATH variable".format(
+                    cdxgen_cmd
+                )
+            )
+            return False
+        try:
+            cdxgen_cmd = local_bin
+            # Set the plugins directory as an environment variable
+            os.environ["CDXGEN_PLUGINS_DIR"] = resource_path("local_bin")
+        except Exception as e:
+            pass
     if project_type in ("docker"):
         LOG.info(
             f"Generating Software Bill-of-Materials for container image {src_dir}. This might take a few mins ..."
         )
-    args = [cdxgen_cmd, "-r", "-t", project_type, "-o", bom_file, src_dir]
+    args = [cdxgen_cmd, "-r", "-t", project_type, "-o", bom_file]
+    if deep or project_type in ("jar", "jenkins"):
+        args.append("--deep")
+        LOG.info("About to perform deep scan. This would take a while ...")
+    args.append(src_dir)
     exec_tool(args)
     return os.path.exists(bom_file)
