@@ -1,29 +1,54 @@
 import os
-from os.path import dirname, join, realpath
+import sys
+from os.path import dirname, exists, join
 
-license_data_dir = join(
-    dirname(realpath(__file__)),
-    "..",
-    "..",
-    "vendor",
-    "choosealicense.com",
-    "_licenses",
-)
 
-spdx_license_list = join(
-    dirname(realpath(__file__)),
-    "..",
-    "..",
-    "vendor",
-    "spdx",
-    "json",
-    "licenses.json",
+def resource_path(relative_path):
+    try:
+        base_path = sys._MEIPASS
+    except Exception:
+        base_path = dirname(__file__)
+    return join(base_path, relative_path)
+
+
+license_data_dir = resource_path(
+    join(
+        "..",
+        "..",
+        "vendor",
+        "choosealicense.com",
+        "_licenses",
+    )
 )
+spdx_license_list = resource_path(
+    join(
+        "..",
+        "..",
+        "vendor",
+        "spdx",
+        "json",
+        "licenses.json",
+    )
+)
+if not exists(license_data_dir):
+    license_data_dir = resource_path(
+        join(
+            "vendor",
+            "choosealicense.com",
+            "_licenses",
+        )
+    )
+    spdx_license_list = resource_path(
+        join(
+            "vendor",
+            "spdx",
+            "json",
+            "licenses.json",
+        )
+    )
 
 # CPE Vendor aliases
 vendor_alias = {
-    "org.apache.tomcat.embed": "apache",
-    "org.apache.tomcat": "apache_tomcat",
     "org.apache.commons.io": "commons-io",
     "org.apache.logging.log4j": "log4j",
     "org.apache.commons.beanutils": "commons-beanutils",
@@ -123,6 +148,7 @@ package_alias = {
     "System.Text": ".net",
     "Microsoft.IdentityModel.Clients.ActiveDirectory": "active_directory_authentication_library",
     "starkbank_ecdsa": "ecdsa-elixir",
+    "php-pear": "pear-core-minimal",
 }
 
 # Default ignore list
@@ -184,6 +210,33 @@ LANG_PKG_TYPES = {
     "github": "github",
 }
 
+# OS Package types
+OS_PKG_TYPES = (
+    "deb",
+    "apk",
+    "rpm",
+    "swid",
+    "alpm",
+    "docker",
+    "oci",
+    "generic",
+    "qpkg",
+    "buildroot",
+    "coreos",
+    "ebuild",
+    "alpine",
+    "alma",
+    "debian",
+    "ubuntu",
+    "amazon",
+    "redhat",
+    "rocky",
+    "archlinux",
+    "suse",
+    "photon",
+    "microsoft",
+)
+
 
 def get_float_from_env(name, default):
     value = os.getenv(name.upper(), default)
