@@ -1,4 +1,6 @@
 from vdb.lib import KNOWN_PKG_TYPES
+from vdb.lib.config import placeholder_exclude_version
+from vdb.lib.utils import parse_purl
 
 from depscan.lib import config as config
 
@@ -40,10 +42,28 @@ def create_pkg_variations(pkg_dict):
     vendor = pkg_dict.get("vendor")
     name = pkg_dict.get("name")
     purl = pkg_dict.get("purl", "")
+    pkg_type = ""
+    name_aliases.add(name)
+    name_aliases.add(name.lower())
+    name_aliases.add(name.replace("-", "_"))
+    os_distro = None
+    os_distro_name = None
     if purl:
-        tmpParts = purl.split(":")
-        if tmpParts and len(tmpParts) > 1:
-            vendor_aliases.add(tmpParts[1])
+        try:
+            purl_obj = parse_purl(purl)
+            if purl_obj:
+                pkg_type = purl_obj.get("type")
+                qualifiers = purl_obj.get("qualifiers", {})
+                if qualifiers.get("distro_name"):
+                    os_distro_name = qualifiers.get("distro_name")
+                    name_aliases.add(f"""{os_distro_name}/{name}""")
+                if qualifiers.get("distro"):
+                    os_distro = qualifiers.get("distro")
+                    name_aliases.add(f"""{os_distro}/{name}""")
+        except Exception:
+            tmpParts = purl.split(":")
+            if tmpParts and len(tmpParts) > 1:
+                vendor_aliases.add(tmpParts[1])
     if vendor:
         vendor_aliases.add(vendor)
         vendor_aliases.add(vendor.lower())
@@ -65,19 +85,20 @@ def create_pkg_variations(pkg_dict):
         # Ignore third party alternatives for builtins
         if "golang" not in vendor and name not in ["net", "crypto", "http", "text"]:
             vendor_aliases.add("golang")
-    if not purl.startswith("pkg:golang"):
-        vendor_aliases.add("get" + name)
-        vendor_aliases.add(name + "_project")
-    for k, v in config.vendor_alias.items():
-        if vendor and (vendor.startswith(k) or k.startswith(vendor)):
-            vendor_aliases.add(k)
-            vendor_aliases.add(v)
-        elif name == k:
-            vendor_aliases.add(v)
-    name_aliases.add(name)
-    name_aliases.add(name.lower())
-    name_aliases.add(name.replace("-", "_"))
-    name_aliases.add("package_" + name)
+    if pkg_type not in config.OS_PKG_TYPES:
+        name_aliases.add("package_" + name)
+        if not purl.startswith("pkg:golang"):
+            vendor_aliases.add("get" + name)
+            vendor_aliases.add(name + "_project")
+        for k, v in config.vendor_alias.items():
+            if vendor and (vendor.startswith(k) or k.startswith(vendor)):
+                vendor_aliases.add(k)
+                vendor_aliases.add(v)
+            elif name == k:
+                vendor_aliases.add(v)
+    # This will add false positives to ubuntu
+    if "/" in name and os_distro and "ubuntu" not in os_distro:
+        name_aliases.add(name.split("/")[-1])
     # Pypi specific vendor aliases
     if purl.startswith("pkg:pypi"):
         if not name.startswith("python-"):
@@ -121,13 +142,21 @@ def create_pkg_variations(pkg_dict):
     elif purl.startswith("pkg:github"):
         vendor_aliases.add("github actions")
         name_aliases.add(f"{vendor}/{name}")
-    for suffix in COMMON_SUFFIXES:
-        if name.endswith(suffix):
-            name_aliases.add(name.replace(suffix, ""))
-    for k, v in config.package_alias.items():
-        if name.startswith(k) or k.startswith(name) or v.startswith(name):
-            name_aliases.add(k)
-            name_aliases.add(v)
+    if pkg_type not in config.OS_PKG_TYPES:
+        for suffix in COMMON_SUFFIXES:
+            if name.endswith(suffix):
+                name_aliases.add(name.replace(suffix, ""))
+        for k, v in config.package_alias.items():
+            if name.startswith(k) or k.startswith(name) or v.startswith(name):
+                name_aliases.add(k)
+                name_aliases.add(v)
+    if pkg_type in config.OS_PKG_TYPES:
+        if "lib" in name:
+            name_aliases.add(name.replace("lib", ""))
+        elif "lib" not in name:
+            name_aliases.add("lib" + name)
+        if "-bin" not in name:
+            name_aliases.add(name + "-bin")
     if len(vendor_aliases):
         for vvar in list(vendor_aliases):
             for nvar in list(name_aliases):
@@ -138,13 +167,14 @@ def create_pkg_variations(pkg_dict):
     return pkg_list
 
 
-def dealias_packages(project_type, pkg_list, pkg_aliases):
+def dealias_packages(project_type, pkg_list, pkg_aliases, purl_aliases):
     """Method to dealias package names by looking up vendor and name information
     in the aliases list
 
     :param project_type: Project type
     :param pkg_list: List of packages to dealias
     :param pkg_aliases: Package aliases
+    :param purl_aliases: Package URL aliases
     """
     if not pkg_aliases:
         return {}
@@ -157,12 +187,16 @@ def dealias_packages(project_type, pkg_list, pkg_aliases):
                 package_issue.affected_location.vendor,
                 package_issue.affected_location.package,
             )
-        for k, v in pkg_aliases.items():
-            if (
-                full_pkg in v or (":" + package_issue.affected_location.package) in v
-            ) and full_pkg != k:
-                dealias_dict[full_pkg] = k
-                break
+        if purl_aliases.get(full_pkg.lower()):
+            dealias_dict[full_pkg] = purl_aliases.get(full_pkg.lower())
+        else:
+            for k, v in pkg_aliases.items():
+                if (
+                    full_pkg in v
+                    or (":" + package_issue.affected_location.package) in v
+                ) and full_pkg != k:
+                    dealias_dict[full_pkg] = k
+                    break
     return dealias_dict
 
 
@@ -179,6 +213,13 @@ def dedup(project_type, pkg_list, pkg_aliases):
         vid = res.id
         vuln_occ_dict = res.to_dict()
         package_type = vuln_occ_dict.get("type")
+        package_issue = res.package_issue
+        fixed_location = package_issue.fixed_location
+        # Ignore any result with the exclude fix location
+        # Required for debian
+        if fixed_location == placeholder_exclude_version:
+            dedup_dict[vid] = True
+            continue
         allowed_type = config.LANG_PKG_TYPES.get(project_type)
         if package_type and package_type in KNOWN_PKG_TYPES and allowed_type:
             if allowed_type != package_type:
